@@ -551,6 +551,21 @@ class Session(object):
             response['error'] = errorstr
         self.lastresponse = response
 
+    def _isincommand(self):
+        if self.incommand:
+            stillin = self.incommand - _monotonic_time()
+            if stillin > 0:
+                return stillin
+        return 0
+
+    def _getmaxtimeout(self):
+        cumulativetime = 0
+        incrementtime = self.timeout
+        while incrementtime < self.maxtimeout:
+            cumulativetime += incrementtime
+            incrementtime += 1
+        return cumulativetime + 1
+
     def raw_command(self,
                     netfn,
                     command,
@@ -560,11 +575,11 @@ class Session(object):
                     delay_xmit=None):
         if not self.logged:
             raise exc.IpmiException('Session no longer connected')
-        while self.incommand:
-            Session.wait_for_rsp()
+        while self._isincommand():
+            Session.wait_for_rsp(self._isincommand())
         if not self.logged:
             raise exc.IpmiException('Session no longer connected')
-        self.incommand = True
+        self.incommand = _monotonic_time() + self._getmaxtimeout()
         self.lastresponse = None
         self.ipmicallback = self._generic_callback
         self._send_ipmi_net_payload(netfn, command, data,
@@ -944,7 +959,9 @@ class Session(object):
         sessionstodel = []
         sessionstokeepalive = []
         for session, parms in cls.keepalive_sessions.iteritems():
-            if parms['timeout'] < curtime:
+            # if the session is busy inside a command, defer invoking keepalive
+            # until incommand is no longer the case
+            if parms['timeout'] < curtime and not session._isincommand():
                 cls.keepalive_sessions[session]['timeout'] = \
                     _monotonic_time() + 25 + (random.random() * 4.9)
                 sessionstokeepalive.append(session)
@@ -1005,7 +1022,15 @@ class Session(object):
             else:
                 kaids = list(self._customkeepalives.keys())
                 for keepalive in kaids:
-                    cmd, callback = self._customkeepalives[keepalive]
+                    try:
+                        cmd, callback = self._customkeepalives[keepalive]
+                    except TypeError:
+                        # raw_command made customkeepalives None
+                        break
+                    except KeyError:
+                        # raw command ultimately caused a keepalive to
+                        # deregister
+                        continue
                     callback(self.raw_command(**cmd))
         except exc.IpmiException:
             self._mark_broken()
@@ -1393,11 +1418,6 @@ class Session(object):
         self.nowait = False
 
     def _xmit_packet(self, retry=True, delay_xmit=None):
-        if not self.nowait:  # if we are retrying, we really need to get the
-                            # packet out and get our timeout updated
-            Session.wait_for_rsp(timeout=0, callout=False)  # take opportunity
-                                                 # to drain the socket queue if
-                                                 # applicable
         if self.sequencenumber:  # seq number of zero will be left alone, it is
                                 # special, otherwise increment
             self.sequencenumber += 1
